unit OpenGLForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, OpenGLScene, ExtCtrls, OpenGLTypes;

type
  TGLForm = class(TForm)
    procedure FormActivate(Sender: TObject); virtual;
    procedure FormClick(Sender: TObject); virtual;
    procedure FormClose(Sender: TObject; var Action: TCloseAction); virtual;
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState); virtual;
    procedure FormKeyPress(Sender: TObject; var Key: Char); virtual;
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState); virtual;
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); virtual;
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); virtual;
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); virtual;
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean); virtual;
    procedure FormMouseWheelDown(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean); virtual;
    procedure FormMouseWheelUp(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean); virtual;
    procedure FormPaint(Sender: TObject); virtual;
    procedure FormResize(Sender: TObject); virtual;
    procedure FormDestroy(Sender: TObject);
    procedure SetOpenGLInitialProperties; virtual; abstract; 
  private
    { Private declarations }
  protected
    { Protected declarations }
    GLScene: TGLScene;
  public
    { Public declarations }
  end;

//var
//  GLForm: TGLForm;

implementation

{$R *.dfm}

procedure TGLForm.FormActivate(Sender: TObject);
begin
//
end;

procedure TGLForm.FormClick(Sender: TObject);
begin
//
end;

procedure TGLForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
//
end;

procedure TGLForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if GLScene <> nil then
    if GLScene.OnKeyDown(Sender, Key, Shift) = glerRepaintNeeded  then
      self.FormPaint(self);
end;

procedure TGLForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if GLScene <> nil then
    if GLScene.OnKeyPress(Sender, Key) = glerRepaintNeeded then
      self.FormPaint(self);
end;

procedure TGLForm.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if GLScene <> nil then
    if GLScene.OnKeyUp(Sender, Key, Shift) = glerRepaintNeeded then
      self.FormPaint(self);
end;

procedure TGLForm.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if GLScene <> nil then
    if GLScene.OnMouseDown(Sender, Button, Shift, X, Y) = glerRepaintNeeded then
      self.FormPaint(self);
end;

procedure TGLForm.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if GLScene <> nil then
    if GLScene.OnMouseMove(Sender, Shift, X, Y) = glerRepaintNeeded then
      self.FormPaint(self);
end;

procedure TGLForm.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if GLScene <> nil then
    if GLScene.OnMouseUp(Sender, Button, Shift, X, Y) = glerRepaintNeeded then
      self.FormPaint(self);
end;

procedure TGLForm.FormMouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
  if GLScene <> nil then
    if GLScene.OnMouseWheel(Sender, Shift, WheelDelta, MousePos, Handled) = glerRepaintNeeded then
      self.FormPaint(self);
end;

procedure TGLForm.FormMouseWheelDown(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  if GLScene <> nil then
    if GLScene.OnMouseWheelDown(Sender, Shift, MousePos, Handled) = glerRepaintNeeded then
      self.FormPaint(self);
end;

procedure TGLForm.FormMouseWheelUp(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  if GLScene <> nil then
    if GLScene.OnMouseWheelUp(Sender, Shift, MousePos, Handled) = glerRepaintNeeded then
      self.FormPaint(self);
end;

procedure TGLForm.FormPaint(Sender: TObject);
begin
  if GLScene <> nil then
    GLScene.DrawObjects;
end;

procedure TGLForm.FormResize(Sender: TObject);
begin
  if GLScene <> nil then
    if GLScene.OnFormResize(Sender) = glerRepaintNeeded then
      self.FormPaint(self);
end;

procedure TGLForm.FormDestroy(Sender: TObject);
begin
  if GLScene <> nil then
    FreeAndNil(GLScene);
end;

end.
