unit OpenGLInterfaceTools;

interface

uses
  OpenGL, Forms, Controls, Windows, Math;

type

  TGLWindowModeSwitchTool = class
  private
    fullScreenMode: Boolean;
    normalModeWidth: Integer;
    normalModeHeight: Integer;
    form: TForm;
    procedure ForceRepaint(width, height: Integer);
  public
    constructor Create(form: TForm);
    procedure SwitchFullScreen;
    procedure SwitchNormal;
    procedure SwitchOpposite;
  end;
        //clip ma miec cztery albo dwa parametry, zawsze zmieniac lastX i lastY na center
        // i ustawiac mysz w centrze.
  TGLMouseAreaClipTool = class
  private
    mouseClipped: Boolean;
    drawArea: TControl;
    clipCenter: TPoint;
    clipArea: TRect;
    oldCursorPos: TPoint;
    //used to calculate clip area when drawing on form, but with gap from one or more sides
    leftGap, topGap, rightGap, bottomGap: Integer;
    procedure CalculateClipParameters;
    procedure SetClipArea;
  public
    constructor Create(drawArea: TControl); overload;
    constructor Create(drawArea: TControl; leftGap, topGap, rightGap, bottomGap: Integer); overload;
    procedure Clip(var newX, newY: Integer); overload;
    procedure Clip(X, Y: Integer; var newX, newY: Integer); overload;
    procedure Release(var lastX, lastY: Integer);
    procedure SetCursor(X, Y: Integer; var lastX, lastY: Integer);
    procedure SetGaps(left, top, right, bottom: Integer);
    procedure UpdateClipArea;
  end;


implementation

{ TGLWindowModeSwitchTool }

constructor TGLWindowModeSwitchTool.Create(form: TForm);
begin
  self.form := form;
end;

procedure TGLWindowModeSwitchTool.ForceRepaint(width, height: Integer);
begin
  //Force a full repaint of a window. InvalidateRect() should do this, but it doesn't work
  //Shrink the window
  SetWindowPos(form.Handle,HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE);
  //Restore its former size
  SetWindowPos(form.Handle,HWND_TOPMOST, 0, 0, width, height, SWP_NOMOVE);
end;

procedure TGLWindowModeSwitchTool.switchFullScreen;
var
  style: longword;
begin
  if not fullScreenMode then
  begin
    //remember form dimensions
    normalModeWidth := form.Width;
    normalModeHeight := form.Height;
    //maximize the window
    form.WindowState := wsMaximized;
    //change window styles
    style := getWindowLong(form.Handle, GWL_STYLE);
    style := (style or WS_POPUP) and (not (WS_BORDER or WS_THICKFRAME or WS_DLGFRAME));
    SetWindowLong(form.handle, GWL_STYLE, style);
    //force Windows to repaint the window, with size of the screen
    //to cover all screen u have to +smth
    ForceRepaint(Screen.Width+4, Screen.Height+4);
    //set mode-indicating variable to fullscreen
    fullScreenMode := true;
  end;
end;

procedure TGLWindowModeSwitchTool.switchNormal;
var
  style: longword;
begin
  if fullScreenMode then
  begin
    //set window state from maximized to normal
    form.WindowState := wsNormal;
    //change window styles
    style := getWindowLong(form.Handle, GWL_STYLE);
    style := (style or WS_POPUP) {and}or ({not} (WS_BORDER or WS_THICKFRAME or WS_DLGFRAME));
    SetWindowLong(form.handle, GWL_STYLE, style);
    //force Windows to repaint the window with its former dimensions
    ForceRepaint(normalModeWidth, normalModeHeight);
    //set mode-indicating variable to normal
    fullScreenMode := false;
  end;
end;

procedure TGLWindowModeSwitchTool.SwitchOpposite;
begin
  if fullScreenMode then
    SwitchNormal
  else
    SwitchFullScreen;
end;

{ TGLMouseAreaClipTool }

procedure TGLMouseAreaClipTool.CalculateClipParameters;
var
  clipWidth, clipHeight: Integer;
begin
  clipWidth := drawArea.ClientWidth - leftGap - rightGap;
  if clipWidth < 0 then clipWidth := 0;
  clipHeight := drawArea.ClientHeight - topGap - bottomGap;
  if clipHeight < 0 then clipHeight := 0;
  //set clip center
  clipCenter.X := leftGap + floor(clipWidth/2);
  clipCenter.Y := topGap + floor(clipHeight/2);        
  //set clip bounds
  clipArea.Left := clipCenter.X - floor(clipWidth/2) + 10;
  clipArea.Top := clipCenter.Y - floor(clipHeight/2) + 10;
  clipArea.Right := clipCenter.X + floor(clipWidth/2) - 10;
  clipArea.Bottom := clipCenter.Y + floor(clipHeight/2) - 10;
end;

procedure TGLMouseAreaClipTool.Clip(X, Y: Integer; var newX, newY: Integer);
begin
  if not mouseClipped then
  begin
    //remember cursor position before clip
    oldCursorPos.X := X;
    oldCursorPos.Y := Y;
    //calculate and set clip area
    SetClipArea;
    //set new X, Y position variables to clip center
    newX := clipCenter.X;
    newY := clipCenter.Y;
    //set boolean variable to indicate that cursor is clipped
    mouseClipped := true;
  end;
end;

procedure TGLMouseAreaClipTool.Clip(var newX, newY: Integer);
begin
  if not mouseClipped then
  begin
    //calculate and set clip area
    SetClipArea;
    //remember center of draw area as a old cursor position
    oldCursorPos.X := clipCenter.X;
    oldCursorPos.Y := clipCenter.Y;
    //set new X, Y position variables to clip center
    newX := clipCenter.X;
    newY := clipCenter.Y;
    //set boolean variable to indicate that cursor is clipped
    mouseClipped := true;
  end;
end;

constructor TGLMouseAreaClipTool.Create(drawArea: TControl);
begin
  self.drawArea := drawArea;
end;

constructor TGLMouseAreaClipTool.Create(drawArea: TControl; leftGap,
  topGap, rightGap, bottomGap: Integer);
begin
  self.drawArea := drawArea;
  self.leftGap := leftGap;
  self.topGap:= topGap;
  self.rightGap := rightGap;
  self.bottomGap := bottomGap;
end;

procedure TGLMouseAreaClipTool.Release(var lastX, lastY: Integer);
begin
  if mouseClipped then
  begin
    //call Windows API function to release cursor
    Windows.ClipCursor(nil);
    //set cursor position to its position before it was clipped
    Mouse.CursorPos := drawArea.ClientToScreen(oldCursorPos);
    //set lastX and lastY to new cursor position
    lastX := oldCursorPos.X;
    lastY := oldCursorPos.Y;
    //set boolean variable to indicate that cursor is no longer clipped
    mouseClipped := false;
  end;
end;

procedure TGLMouseAreaClipTool.SetClipArea;
var
  pTopLeft, pBottomRight: TPoint;
  cursorClipArea: TRect;
begin
  //calculate clip center and clip area
  CalculateClipParameters;
  //set cursor in the calculated center
  Mouse.CursorPos := drawArea.ClientToScreen(clipCenter);
  //copy coordinates of clip area to points variables
  pTopLeft.X := clipArea.Left-5;
  pTopLeft.Y := clipArea.Top-5;
  pBottomRight.X := clipArea.Right+5;
  pBottomRight.Y := clipArea.Bottom+5;
  //change values from client to screen coordinates
  pTopLeft := drawArea.ClientToScreen(pTopLeft);
  pBottomRight := drawArea.ClientToScreen(pBottomRight);
  //copy screen coordinates to TRect variable
  cursorClipArea.Left := pTopLeft.X;
  cursorClipArea.Top := pTopLeft.Y;
  cursorClipArea.Right := pBottomRight.X;
  cursorClipArea.Bottom := pBottomRight.Y;
  //call Windows API function to set cursor clip area
  Windows.ClipCursor(@cursorClipArea);
end;

procedure TGLMouseAreaClipTool.SetCursor(X, Y: Integer; var lastX, lastY: Integer);
begin
  if (X < clipArea.Left) or (X > clipArea.Right) or
     (Y < clipArea.Top) or (Y > clipArea.Bottom) then
  begin
    Mouse.CursorPos := drawArea.ClientToScreen(clipCenter);
    lastX := clipCenter.X;
    lastY := clipCenter.Y;
  end
  else
  begin
    lastX := X;
    lastY := Y;
  end;
end;

procedure TGLMouseAreaClipTool.SetGaps(left, top, right, bottom: Integer);
begin
  self.leftGap := left;
  self.topGap := top;
  self.rightGap := right;
  self.bottomGap := bottom;
end;

procedure TGLMouseAreaClipTool.UpdateClipArea;
begin
  if mouseClipped then
    SetClipArea;
end;

end.
