//*****************************************************************************
// File:        Point.pas
//
// Classe:      TPoint
//
// Description: TPoint represents one point in 3d: x-y-z plus attribute.
//
// Note:
//
//*****************************************************************************
// Last version: 17-08-2005
//*****************************************************************************

unit Point;

interface

type
  TPoint = class
  private
    m_x:        double;
    m_y:        double;
    m_z:        double;
    m_data:     double;
    m_flag:     boolean; //-- to draw some points in different colours
    procedure   SetX(x: double);
    procedure   SetY(y: double);
    procedure   SetZ(z: double);
    procedure   SetData(data: double);
    procedure   SetFlag(b: boolean);
  public
    m_no:       integer;  { TODO : remove this (m_no in TPoint) }
    constructor Create(x: double = 0.0; y: double = 0.0; z: double = 0.0;
                       data: double = 0.0); overload;
    constructor Create(flag: boolean; x: double = 0.0; y: double = 0.0; z: double = 0.0;
                       data: double = 0.0); overload;
    procedure   SetCoord(x, y, z: double);
    function    GetCoord : TPoint;
    procedure   CopyValue(tempt: TPoint);
    function    Hit(const pt: TPoint; tolerance: double) : boolean;
  published
    property    x: double read m_x write SetX;
    property    y: double read m_y write SetY;
    property    z: double read m_z write SetZ;
    property    data: double read m_data write SetData;
    property    flag: boolean read m_flag write SetFlag;
  end;


implementation

uses
  GeomTools;

var
  no: integer = 0;


{ TPoint }

constructor TPoint.Create(x, y, z, data: double);
begin
  SetCoord(x, y, z);
  m_data := data;
  m_no := 0;
  m_flag := false;
end;


procedure TPoint.CopyValue(tempt: TPoint);
begin
  m_x := tempt.x;
  m_y := tempt.y;
  m_z := tempt.z;
  m_data := tempt.data;
end;

constructor TPoint.Create(flag: boolean; x, y, z, data: double);
begin
  SetCoord(x, y, z);
  m_data := data;
  m_no := no;
  inc(no);
end;


function TPoint.GetCoord: TPoint;
begin
  Result := Self;
end;


//******************************************************************************
// Description: Determine if a point if too close (according to a certain
//              tolerance) from the current point (self)
//
// Input:       pt : the point to test
//              tolerance : the distance in 3D
//
// Output:      TRUE  -> distance self-pt is shorter or equal to the tolerance
//              FALSE -> distance is longer than the tolerance
//******************************************************************************
function TPoint.Hit(const pt: TPoint; tolerance: double): boolean;
begin
  if (TGeomTools.Distance3d(self, pt) <= tolerance) then
  begin
    Result := true;
  end
  else
  begin
    Result := false;
  end;
end;


procedure TPoint.SetCoord(x, y, z: double);
begin
  m_x := x;
  m_y := y;
  m_z := z;
end;

procedure TPoint.SetData(data: double);
begin
  m_data := data;
end;

procedure TPoint.SetX(x: double);
begin
  m_x := x;
end;

procedure TPoint.SetY(y: double);
begin
  m_y := y;
end;

procedure TPoint.SetZ(z: double);
begin
  m_z := z;
end;

procedure TPoint.SetFlag(b: boolean);
begin
  m_flag := b;
end;



end.
