//*****************************************************************************
// File:        Triangle.pas
//
// Classe:      TTriangle
//
// Description: TTriangle represents one triangle in 3d; it manages its own
//              memory. It's only used to draw isosurfaces in the application.
//
// Note:        TTriangle is creating new TPoint because the point are not
//              permanent (they are not part of the mesh, only intermediate
//              points in a tetra, e.g. for faces of an isosurface).
//              TTriangle manages its own memory.
//
//*****************************************************************************
// Last version: 25-06-2002
//*****************************************************************************

unit Triangle;

interface

uses
  Point,
  Vector,
  GeomTools;

type
  TTriangle = class
  private
    m_points:   array[1..3] of TPoint;
    m_normale:  TVector;
  public
    constructor Create(ax, ay, az, bx, by, bz, cx, cy, cz, nx, ny, nz: double); overload;
    constructor Create(a, b, c: TPoint; n: TVector); overload;
    destructor  Destroy; override;
    procedure   SetPoint(i: integer; pt: TPoint);
    function    GetPoint(i: integer) : TPoint;
    procedure   SetNormale(x, y, z: double);
    function    GetNormale : TVector;
  end;


implementation

{ TTriangle }

constructor TTriangle.Create(ax, ay, az, bx, by, bz, cx, cy, cz, nx, ny, nz: double);
begin
  m_points[1] := TPoint.Create(ax, ay, az);
  m_points[2] := TPoint.Create(bx, by, bz);
  m_points[3] := TPoint.Create(cx, cy, cz);
  m_normale := TVector.Create(nx, ny, nz);
end;


constructor TTriangle.Create(a, b, c: TPoint; n: TVector);
begin
  m_points[1] := TPoint.Create(a.x, a.y, a.z);
  m_points[2] := TPoint.Create(b.x, b.y, b.z);
  m_points[3] := TPoint.Create(c.x, c.y, c.z);
  m_normale := TVector.Create(n.coord);

end;

destructor TTriangle.Destroy;
begin
  m_points[1].Free;
  m_points[2].Free;
  m_points[3].Free;
  m_normale.Free;

  inherited Destroy;
end;


function TTriangle.GetNormale: TVector;
begin
  Result := m_normale;
end;


procedure TTriangle.SetNormale(x, y, z: double);
begin
  m_normale.coord.x := x;
  m_normale.coord.y := y;
  m_normale.coord.z := z;
end;


function TTriangle.GetPoint(i: integer): TPoint;
begin
  Result := m_points[i];
end;


procedure TTriangle.SetPoint(i: integer; pt: TPoint);
begin
  m_points[i] := pt;
end;


end.
